import React from 'react';
import {observer} from "mobx-react-lite";
import {makeStyles} from "@material-ui/core/styles";
import {green, red} from "@material-ui/core/colors";
import {t} from "@lingui/macro";
import {RxModuleComponentModel} from "../../../stores/RxModuleComponentStore";
import {useSnackbar} from 'notistack';
import {CurrentUserStore} from "@rx-suite/stores/build";
import {CurrentUserModel} from "@rx-suite/stores";
import {ButtonField} from "@rx-suite/ui";
import {CloudDownload} from "@material-ui/icons";
import {LicenceInfoStore} from "../../../stores/LicenceInfoStore";
import {VersionSelect} from "./VersionSelect";
import {compareVersions} from "compare-versions";
import {Box} from "@material-ui/core";

const useStyles = makeStyles(() => ({
    enabledIcon: {
        color: green[500]
    },
    warningIcon: {
        color: red[500]
    },
    errorLabel: {
        whiteSpace: 'pre-line',
        wordWrap: 'break-word'
    }
}))

type Props = {
    model: RxModuleComponentModel,
    showSelect?: boolean

}

export const UpdateCell: React.FunctionComponent<Props> = observer(({model, showSelect = false}) => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();
    const licence = LicenceInfoStore.selectedModel;
    const [selectedVersion, setSelectedVersion] = React.useState<string>(model.highestAvailableUpdate as string);

    const handleVersionChange = React.useCallback((_, value: string) => {
        setSelectedVersion(value);
    }, []);

    React.useEffect(() => {
        if(model.highestAvailableUpdate) {
            setSelectedVersion(model.highestAvailableUpdate);
        }
    }, [model.highestAvailableUpdate]);

    const onClick = React.useCallback(async e => {
        e.stopPropagation();
        if (window.confirm(`${t`Perform update now?  The module will be unavailable until the update is complete.  This may take several minutes.`} 
        
        ${t`Module:`} ${model.module_name} 
        ${t`Current Version:`} ${model.version}
        ${t`New Version:`} ${selectedVersion}`)) {
            enqueueSnackbar(`${model.module_name}: ${t`Update downloading & transferring`}`, {variant: "info"})
            model.update(selectedVersion).then(resp => {
                if (resp.error) {
                    enqueueSnackbar(`${model.module_name}: ${t`Update failed`} (${resp.error_details || ''}) `, {variant: "error"})
                } else {
                    enqueueSnackbar(`${model.module_name}: ${t`Update applying - the module may take several minutes to restart`}    `, {variant: "success"})
                }
            });
        }
    }, [enqueueSnackbar, model, selectedVersion]);

    const user = CurrentUserStore.selectedModel as unknown as CurrentUserModel;
    const canUpdate = user?.canModule('update', 'apply', model.module_id);
    if (!canUpdate) {
        return <div>{model.highestAvailableUpdate}</div>;
    }

    let text;
    if (model.reachable && selectedVersion && model.version) {
        const comp = compareVersions(selectedVersion, model.version)
        if(comp > 0) {
            text = `${t`Update to`} ${selectedVersion}`;
        } else if(comp === 0) {
            if(showSelect) {
                text = `${t`Reinstall`} ${model.version}`;
            } else {
                return <div>{t`Up To Date`}</div>;
            }

        } else if(comp < 0) {
            text = `${t`Downgrade to`} ${selectedVersion}`;
        }
    } else {
        if(model.updateAvailable) {
            text = `${t`[Offline]`} ${model.highestAvailableUpdate}`;
        } else {
            return <div>{t`Up To Date`}</div>;
        }
    }

    let tooltip: string | undefined;
    if (!licence?.can_download_updates) {
        tooltip = t`Your license does not allow download updates. Please contact support.`
    }

    return (
        <Box display={"flex"}>
            { canUpdate && showSelect && <VersionSelect module={model.module_id} value={selectedVersion} onChange={handleVersionChange} /> }
            <ButtonField
                fullWidth={true}
                onClick={onClick}
                startIcon={<CloudDownload className={model.reachable ? classes.enabledIcon : classes.warningIcon}/>}
                text={text}
                variant={"outlined"}
                loading={model.is_updating}
                disabled={!model.reachable || !licence?.can_download_updates}
                tooltip={tooltip}
            />
        </Box>

    )
});